<a name="g2"></a>

## g2 ⇒ [<code>g2</code>](#g2)
Create a 2D graphics command queue object. Call without using 'new'.

**Kind**: global typedef  
**Link**: https://github.com/goessner/g2  
**Author**: Stefan Goessner  
**License**: MIT License  

| Param | Type | Description |
| --- | --- | --- |
| [opts] | <code>object</code> | Custom options object. |

**Example**  
```js
const ctx = document.getElementById("c").getContext("2d");
g2()                                   // Create 'g2' instance.
    .lin({x1:50,y1:50,x2:100,y2:100})  // Append ...
    .lin({x1:100,y1:100,x2:200,y2:50}) // ... commands.
    .exe(ctx);                         // Execute commands addressing canvas context.
```

* [g2](#g2) ⇒ [<code>g2</code>](#g2)
    * [.clr()](#g2+clr) ⇒ <code>object</code>
    * [.view()](#g2+view) ⇒ <code>object</code>
    * [.grid()](#g2+grid) ⇒ <code>object</code>
    * [.cir()](#g2+cir) ⇒ <code>object</code>
    * [.ell()](#g2+ell) ⇒ <code>object</code>
    * [.arc()](#g2+arc) ⇒ <code>object</code>
    * [.rec()](#g2+rec) ⇒ <code>object</code>
    * [.lin()](#g2+lin) ⇒ <code>object</code>
    * [.ply()](#g2+ply) ⇒ <code>object</code>
    * [.txt()](#g2+txt) ⇒ <code>object</code>
    * [.use()](#g2+use) ⇒ <code>object</code>
    * [.img()](#g2+img) ⇒ <code>object</code>
    * [.beg()](#g2+beg) ⇒ <code>object</code>
    * [.end()](#g2+end) ⇒ <code>object</code>
    * [.p()](#g2+p) ⇒ <code>object</code>
    * [.z()](#g2+z) ⇒ <code>object</code>
    * [.m()](#g2+m) ⇒ <code>object</code>
    * [.l()](#g2+l) ⇒ <code>object</code>
    * [.q()](#g2+q) ⇒ <code>object</code>
    * [.c()](#g2+c) ⇒ <code>object</code>
    * [.a()](#g2+a) ⇒ <code>object</code>
    * [.stroke()](#g2+stroke) ⇒ <code>object</code>
    * [.fill()](#g2+fill) ⇒ <code>object</code>
    * [.drw()](#g2+drw) ⇒ <code>object</code>
    * [.del()](#g2+del) ⇒ <code>object</code>
    * [.ins()](#g2+ins) ⇒ <code>object</code>
    * [.exe(ctx)](#g2+exe) ⇒ <code>object</code>

<a name="g2+clr"></a>

### g2.clr() ⇒ <code>object</code>
Clear viewport region.<br>

**Kind**: instance method of [<code>g2</code>](#g2)  
**Returns**: <code>object</code> - g2  
<a name="g2+view"></a>

### g2.view() ⇒ <code>object</code>
Set the view by placing origin coordinates and scaling factor in device units
and make viewport cartesian.

**Kind**: instance method of [<code>g2</code>](#g2)  
**Returns**: <code>object</code> - g2  

| Type | Description |
| --- | --- |
| <code>object</code> | view arguments object. |

**Properties**

| Name | Type | Default | Description |
| --- | --- | --- | --- |
| [scl] | <code>number</code> | <code>1</code> | absolute scaling factor. |
| [x] | <code>number</code> | <code>0</code> | x-origin in device units. |
| [y] | <code>number</code> | <code>0</code> | y-origin in device units. |
| [cartesian] | <code>boolean</code> | <code>false</code> | set cartesian flag. |

<a name="g2+grid"></a>

### g2.grid() ⇒ <code>object</code>
Draw grid.

**Kind**: instance method of [<code>g2</code>](#g2)  
**Returns**: <code>object</code> - g2  

| Type | Description |
| --- | --- |
| <code>object</code> | grid arguments object. |

**Properties**

| Name | Type | Default | Description |
| --- | --- | --- | --- |
| [color] | <code>string</code> | <code>&quot;#ccc&quot;</code> | change color. |
| [size] | <code>number</code> | <code>20</code> | change space between lines. |

<a name="g2+cir"></a>

### g2.cir() ⇒ <code>object</code>
Draw circle by center and radius.

**Kind**: instance method of [<code>g2</code>](#g2)  
**Returns**: <code>object</code> - g2  

| Type | Description |
| --- | --- |
| <code>object</code> | circle arguments object. |

**Properties**

| Name | Type | Default | Description |
| --- | --- | --- | --- |
| x | <code>number</code> |  | x-value center. |
| y | <code>number</code> |  | y-value center. |
| r | <code>number</code> |  | radius. |
| w | <code>number</code> |  | angle. |
| [fs] | <code>string</code> | <code>&quot;transparent&quot;</code> | fill color. |
| [ls] | <code>string</code> | <code>&quot;black&quot;</code> | stroke color. |
| [lw] | <code>string</code> | <code>1</code> | line width. |
| [sh] | <code>array</code> | <code>[0,0,0,&#x27;transparent&#x27;]</code> | shadow values [`x-offset`,`y-offset`,`blur`,`color`], |

**Example**  
```js
g2().cir({x:100,y:80,r:20})  // Draw circle.
```
<a name="g2+ell"></a>

### g2.ell() ⇒ <code>object</code>
Draw ellipse by center and radius for x and y.

**Kind**: instance method of [<code>g2</code>](#g2)  
**Returns**: <code>object</code> - g2  

| Type | Description |
| --- | --- |
| <code>object</code> | ellispe argument object. |

**Properties**

| Name | Type | Default | Description |
| --- | --- | --- | --- |
| x | <code>number</code> |  | x-value center. |
| y | <code>number</code> |  | y-value center. |
| rx | <code>number</code> |  | radius x-axys. |
| ry | <code>number</code> |  | radius y-axys. |
| w | <code>number</code> |  | start angle. |
| dw | <code>number</code> |  | angular range. |
| rot | <code>number</code> |  | rotation. |
| [fs] | <code>string</code> | <code>&quot;transparent&quot;</code> | fill color. |
| [ls] | <code>string</code> | <code>&quot;black&quot;</code> | stroke color. |
| [lw] | <code>string</code> | <code>1</code> | line width. |
| [ld] | <code>array</code> | <code>[]</code> | line dash array. |
| [sh] | <code>array</code> | <code>[0,0,0,&quot;transparent&quot;]</code> | shadow values [`x-offset`,`y-offset`,`blur`,`color`], |

**Example**  
```js
g2().ell({x:100,y:80,rx:20,ry:30,w:0,dw:2*Math.PI/4,rot:1})  // Draw circle.
```
<a name="g2+arc"></a>

### g2.arc() ⇒ <code>object</code>
Draw arc by center point, radius, start angle and angular range.

**Kind**: instance method of [<code>g2</code>](#g2)  
**Returns**: <code>object</code> - g2  

| Type | Description |
| --- | --- |
| <code>object</code> | arc arguments object. |

**Properties**

| Name | Type | Default | Description |
| --- | --- | --- | --- |
| x | <code>number</code> |  | x-value center. |
| y | <code>number</code> |  | y-value center. |
| r | <code>number</code> |  | radius. |
| [w] | <code>number</code> | <code>0</code> | start angle (in radian). |
| [dw] | <code>number</code> | <code>2*pi</code> | angular range in Radians. |
| [fs] | <code>string</code> | <code>&quot;transparent&quot;</code> | fill color. |
| [ls] | <code>string</code> | <code>&quot;black&quot;</code> | stroke color. |
| [lw] | <code>string</code> | <code>1</code> | line width. |
| [lc] | <code>string</code> | <code>&quot;butt&quot;</code> | line cap [`butt`, `round`, `square`]. |
| [ld] | <code>array</code> | <code>[]</code> | line dash array. |
| [sh] | <code>array</code> | <code>[0,0,0,&quot;transparent&quot;]</code> | shadow values [`x-offset`,`y-offset`,`blur`,`color`], |

**Example**  
```js
g2().arc({x:300,y:400,r:390,w:-Math.PI/4,dw:-Math.PI/2})
    .exe(ctx);
```
<a name="g2+rec"></a>

### g2.rec() ⇒ <code>object</code>
Draw rectangle by anchor point and dimensions.

**Kind**: instance method of [<code>g2</code>](#g2)  
**Returns**: <code>object</code> - g2  

| Type | Description |
| --- | --- |
| <code>object</code> | rectangle arguments object. |

**Properties**

| Name | Type | Default | Description |
| --- | --- | --- | --- |
| x | <code>number</code> |  | x-value upper left corner. |
| y | <code>number</code> |  | y-value upper left corner. |
| b | <code>number</code> |  | width. |
| h | <code>number</code> |  | height. |
| [fs] | <code>string</code> | <code>&quot;transparent&quot;</code> | fill color. |
| [ls] | <code>string</code> | <code>&quot;black&quot;</code> | stroke color. |
| [lw] | <code>string</code> | <code>1</code> | line width. |
| [lj] | <code>string</code> | <code>&quot;&#x27;miter&#x27;&quot;</code> | line join [`round`, `bevel`, `miter`]. |
| [ml] | <code>number</code> | <code>10</code> | miter limit. |
| [ld] | <code>array</code> | <code>[]</code> | line dash array. |
| [sh] | <code>array</code> | <code>[0,0,0,&quot;transparent&quot;]</code> | shadow values [`x-offset`,`y-offset`,`blur`,`color`], |

**Example**  
```js
g2().rec({x:100,y:80,b:40,h:30}) // Draw rectangle.
```
<a name="g2+lin"></a>

### g2.lin() ⇒ <code>object</code>
Draw line by start point and end point.

**Kind**: instance method of [<code>g2</code>](#g2)  
**Returns**: <code>object</code> - g2  

| Type | Description |
| --- | --- |
| <code>object</code> | line arguments object. |

**Properties**

| Name | Type | Default | Description |
| --- | --- | --- | --- |
| x1 | <code>number</code> |  | start x coordinate. |
| y1 | <code>number</code> |  | start y coordinate. |
| x2 | <code>number</code> |  | end x coordinate. |
| y2 | <code>number</code> |  | end y coordinate. |
| [ls] | <code>string</code> | <code>&quot;black&quot;</code> | stroke color. |
| [lw] | <code>string</code> | <code>1</code> | line width. |
| [lc] | <code>string</code> | <code>&quot;butt&quot;</code> | line cap [`butt`, `round`, `square`]. |
| [ld] | <code>array</code> | <code>[]</code> | line dash array. |
| [sh] | <code>array</code> | <code>[0,0,0,&quot;transparent&quot;]</code> | shadow values [`x-offset`,`y-offset`,`blur`,`color`], |

**Example**  
```js
g2().lin({x1:10,x2:10,y1:190,y2:10}) // Draw line.
```
<a name="g2+ply"></a>

### g2.ply() ⇒ <code>object</code>
Draw polygon by points.
Using iterator function for getting points from array by index.
It must return current point object {x,y} or object {done:true}.
Default iterator expects sequence of x/y-coordinates as a flat array [x,y,...],
array of [[x,y],...] arrays or array of [{x,y},...] objects.

**Kind**: instance method of [<code>g2</code>](#g2)  
**Returns**: <code>object</code> - g2  

| Type | Description |
| --- | --- |
| <code>object</code> | polygon arguments object. |

**Properties**

| Name | Type | Default | Description |
| --- | --- | --- | --- |
| pts | <code>array</code> |  | array of points. |
| [format] | <code>string</code> |  | format string of points array structure. Useful for handing over initial empty points array. One of `['x,y','[x,y]','{x,y}']`. Has precedence over `pts` content. |
| [closed] | <code>boolean</code> | <code>false</code> |  |
| x | <code>number</code> |  | start x coordinate. |
| y | <code>number</code> |  | start y coordinate. |
| w | <code>number</code> |  | angle. |
| [fs] | <code>string</code> | <code>&quot;transparent&quot;</code> | fill color. |
| [ls] | <code>string</code> | <code>&quot;black&quot;</code> | stroke color. |
| [lw] | <code>string</code> | <code>1</code> | line width. |
| [lc] | <code>string</code> | <code>&quot;butt&quot;</code> | line cap [`butt`, `round`, `square`]. |
| [lj] | <code>string</code> | <code>&quot;&#x27;miter&#x27;&quot;</code> | line join [`round`, `bevel`, `miter`]. |
| [ml] | <code>number</code> | <code>10</code> | miter limit. |
| [ld] | <code>array</code> | <code>[]</code> | line dash array. |
| [sh] | <code>array</code> | <code>[0,0,0,&quot;transparent&quot;]</code> | shadow values [`x-offset`,`y-offset`,`blur`,`color`], |

**Example**  
```js
g2().ply({pts:[100,50,120,60,80,70]}),
    .ply({pts:[150,60],[170,70],[130,80]],closed:true}),
    .ply({pts:[{x:160,y:70},{x:180,y:80},{x:140,y:90}]}),
    .exe(ctx);
```
<a name="g2+txt"></a>

### g2.txt() ⇒ <code>object</code>
Draw text string at anchor point.

**Kind**: instance method of [<code>g2</code>](#g2)  
**Returns**: <code>object</code> - g2  

| Type | Description |
| --- | --- |
| <code>object</code> | text arguments object. |

**Properties**

| Name | Type | Default | Description |
| --- | --- | --- | --- |
| str | <code>string</code> |  | text string. |
| [x] | <code>number</code> | <code>0</code> | x coordinate of text anchor position. |
| [y] | <code>number</code> | <code>0</code> | y coordinate of text anchor position. |
| [w] | <code>number</code> | <code>0</code> | w Rotation angle about anchor point with respect to positive x-axis. |
| [fs] | <code>string</code> | <code>&quot;transparent&quot;</code> | fill color. |
| [ls] | <code>string</code> | <code>&quot;black&quot;</code> | stroke color. |
| [sh] | <code>array</code> | <code>[0,0,0,&quot;transparent&quot;]</code> | shadow values [`x-offset`,`y-offset`,`blur`,`color`], |
| [thal] | <code>string</code> | <code>&quot;&#x27;start&#x27;&quot;</code> | - Text horizontal alignment [`'start'`,`'end'`,`'left'`,`'right'`,`'center'`] |
| [tval] | <code>string</code> | <code>&quot;&#x27;alphabetic&#x27;&quot;</code> | - Text vertival alignment [`'top'`,`'hanging'`,`'middle'`,`'alphabetic'`,`'ideographic'`,`'bottom'`] |
| [font] | <code>string</code> | <code>&quot;&#x27;normal 14px serif&#x27;&quot;</code> | [Font](https://developer.mozilla.org/en-US/docs/Web/API/CanvasRenderingContext2D/font) [styling](https://html.spec.whatwg.org/multipage/canvas.html#dom-context-2d-font) |

<a name="g2+use"></a>

### g2.use() ⇒ <code>object</code>
Reference g2 graphics commands from another g2 object or a predefined g2.symbol.
With this command you can reuse instances of grouped graphics commands
while applying a similarity transformation and style properties on them.
In fact you might want to build custom graphics libraries on top of that feature.

**Kind**: instance method of [<code>g2</code>](#g2)  
**Returns**: <code>object</code> - g2  
**See**: [predefined symbols in g2.ext](https://github.com/goessner/g2/blob/master/docs/api/g2.ext.md#g2symbol--object)  

| Type | Description |
| --- | --- |
| <code>object</code> | use arguments object. |

**Properties**

| Name | Type | Default | Description |
| --- | --- | --- | --- |
| grp | <code>object</code> \| <code>string</code> |  | g2 source object or symbol name found in 'g2.symbol' namespace. |
| [x] | <code>number</code> | <code>0</code> | translation value x. |
| [y] | <code>number</code> | <code>0</code> | translation value y. |
| [w] | <code>number</code> | <code>0</code> | rotation angle (in radians). |
| [scl] | <code>number</code> | <code>1</code> | scale factor. |
| [fs] | <code>string</code> | <code>&quot;transparent&quot;</code> | fill color. |
| [ls] | <code>string</code> | <code>&quot;black&quot;</code> | stroke color. |
| [lw] | <code>string</code> | <code>1</code> | line width. |
| [lc] | <code>string</code> | <code>&quot;butt&quot;</code> | line cap [`butt`, `round`, `square`]. |
| [lj] | <code>string</code> | <code>&quot;&#x27;miter&#x27;&quot;</code> | line join [`round`, `bevel`, `miter`]. |
| [ml] | <code>number</code> | <code>10</code> | miter limit. |
| [ld] | <code>array</code> | <code>[]</code> | line dash array. |
| [sh] | <code>array</code> | <code>[0,0,0,&quot;transparent&quot;]</code> | shadow values [`x-offset`,`y-offset`,`blur`,`color`], |
| [thal] | <code>string</code> | <code>&quot;&#x27;start&#x27;&quot;</code> | - Text horizontal alignment [`'start'`,`'end'`,`'left'`,`'right'`,`'center'`] |
| [tval] | <code>string</code> | <code>&quot;&#x27;alphabetic&#x27;&quot;</code> | - Text vertival alignment [`'top'`,`'hanging'`,`'middle'`,`'alphabetic'`,`'ideographic'`,`'bottom'`] |
| [font] | <code>string</code> | <code>&quot;&#x27;normal 14px serif&#x27;&quot;</code> | [Font](https://developer.mozilla.org/en-US/docs/Web/API/CanvasRenderingContext2D/font) [styling](https://html.spec.whatwg.org/multipage/canvas.html#dom-context-2d-font) |

**Example**  
```js
g2.symbol.cross = g2().lin({x1:5,y1:5,x2:-5,y2:-5}).lin({x1:5,y1:-5,x2:-5,y2:5});  // Define symbol.
g2().use({grp:"cross",x:100,y:100})  // Draw cross at position 100,100.
```
<a name="g2+img"></a>

### g2.img() ⇒ <code>object</code>
Draw image.
This also applies to images of reused g2 objects. If an image can not be loaded, it will be replaced by a broken-image symbol.

**Kind**: instance method of [<code>g2</code>](#g2)  
**Returns**: <code>object</code> - g2  

| Type | Description |
| --- | --- |
| <code>object</code> | image arguments object. |

**Properties**

| Name | Type | Default | Description |
| --- | --- | --- | --- |
| uri | <code>string</code> |  | image uri or data:url. |
| [x] | <code>number</code> | <code>0</code> | x-coordinate of image (upper left). |
| [y] | <code>number</code> | <code>0</code> | y-coordinate of image (upper left). |
| [b] | <code>number</code> | <code>image.width</code> | width. |
| [h] | <code>number</code> | <code>image.height</code> | height. |
| [sx] | <code>number</code> | <code>0</code> | source x-offset. |
| [sy] | <code>number</code> | <code>0</code> | source y-offset. |
| [sb] | <code>number</code> | <code>image.width</code> | source width. |
| [sh] | <code>number</code> | <code>image.height</code> | source height. |
| [xoff] | <code>number</code> | <code>0</code> | x-offset. |
| [yoff] | <code>number</code> | <code>0</code> | y-offset. |
| [w] | <code>number</code> | <code>0</code> | rotation angle (about upper left, in radians). |
| [scl] | <code>number</code> | <code>1</code> | image scaling. |

<a name="g2+beg"></a>

### g2.beg() ⇒ <code>object</code>
Begin subcommands. Current state is saved.
Optionally apply transformation or style properties.

**Kind**: instance method of [<code>g2</code>](#g2)  
**Returns**: <code>object</code> - g2  

| Type | Description |
| --- | --- |
| <code>object</code> | beg arguments object. |

**Properties**

| Name | Type | Default | Description |
| --- | --- | --- | --- |
| [x] | <code>number</code> | <code>0</code> | translation value x. |
| [y] | <code>number</code> | <code>0</code> | translation value y. |
| [w] | <code>number</code> | <code>0</code> | rotation angle (in radians). |
| [scl] | <code>number</code> | <code>1</code> | scale factor. |
| [matrix] | <code>array</code> |  | matrix instead of single transform arguments (SVG-structure [a,b,c,d,x,y]). |
| [fs] | <code>string</code> | <code>&quot;transparent&quot;</code> | fill color. |
| [ls] | <code>string</code> | <code>&quot;black&quot;</code> | stroke color. |
| [lw] | <code>string</code> | <code>1</code> | line width. |
| [lc] | <code>string</code> | <code>&quot;butt&quot;</code> | line cap [`butt`, `round`, `square`]. |
| [lj] | <code>string</code> | <code>&quot;&#x27;miter&#x27;&quot;</code> | line join [`round`, `bevel`, `miter`]. |
| [ml] | <code>number</code> | <code>10</code> | miter limit. |
| [ld] | <code>array</code> | <code>[]</code> | line dash array. |
| [sh] | <code>array</code> | <code>[0,0,0,&quot;transparent&quot;]</code> | shadow values [`x-offset`,`y-offset`,`blur`,`color`], |
| [thal] | <code>string</code> | <code>&quot;&#x27;start&#x27;&quot;</code> | - text horizontal alignment [`'start'`,`'end'`,`'left'`,`'right'`,`'center'`] |
| [tval] | <code>string</code> | <code>&quot;&#x27;alphabetic&#x27;&quot;</code> | - text vertival alignment [`'top'`,`'hanging'`,`'middle'`,`'alphabetic'`,`'ideographic'`,`'bottom'`] |
| [font] | <code>string</code> | <code>&quot;&#x27;normal 14px serif&#x27;&quot;</code> | [Font](https://developer.mozilla.org/en-US/docs/Web/API/CanvasRenderingContext2D/font) [styling](https://html.spec.whatwg.org/multipage/canvas.html#dom-context-2d-font) |

<a name="g2+end"></a>

### g2.end() ⇒ <code>object</code>
End subcommands. Previous state is restored.

**Kind**: instance method of [<code>g2</code>](#g2)  
**Returns**: <code>object</code> - g2  

| Type | Description |
| --- | --- |
| <code>object</code> | end arguments object. |

<a name="g2+p"></a>

### g2.p() ⇒ <code>object</code>
Begin new path.

**Kind**: instance method of [<code>g2</code>](#g2)  
**Returns**: <code>object</code> - g2  
<a name="g2+z"></a>

### g2.z() ⇒ <code>object</code>
Close current path by straight line.

**Kind**: instance method of [<code>g2</code>](#g2)  
**Returns**: <code>object</code> - g2  
<a name="g2+m"></a>

### g2.m() ⇒ <code>object</code>
Move to point.

**Kind**: instance method of [<code>g2</code>](#g2)  
**Returns**: <code>object</code> - g2  

| Type | Description |
| --- | --- |
| <code>object</code> | move arguments object. |

**Properties**

| Name | Type | Description |
| --- | --- | --- |
| x | <code>number</code> | move to x coordinate |
| y | <code>number</code> | move to y coordinate |

<a name="g2+l"></a>

### g2.l() ⇒ <code>object</code>
Create line segment to point.

**Kind**: instance method of [<code>g2</code>](#g2)  
**Returns**: <code>object</code> - g2  

| Type | Description |
| --- | --- |
| <code>object</code> | line segment argument object. |

**Properties**

| Name | Type | Description |
| --- | --- | --- |
| x | <code>number</code> | x coordinate of target point. |
| y | <code>number</code> | y coordinate of target point. |

**Example**  
```js
g2().p()             // Begin path.
    .m({x:0,y:50})   // Move to point.
    .l({x:300,y:0})  // Line segment to point.
    .l({x:400,y:100}) // ...
    .stroke()        // Stroke path.
```
<a name="g2+q"></a>

### g2.q() ⇒ <code>object</code>
Create quadratic bezier curve segment to point.

**Kind**: instance method of [<code>g2</code>](#g2)  
**Returns**: <code>object</code> - g2  

| Type | Description |
| --- | --- |
| <code>object</code> | quadratic curve arguments object. |

**Properties**

| Name | Type | Description |
| --- | --- | --- |
| x1 | <code>number</code> | x coordinate of control point. |
| y1 | <code>number</code> | y coordinate of control point. |
| x | <code>number</code> | x coordinate of target point. |
| y | <code>number</code> | y coordinate of target point. |

**Example**  
```js
g2().p()                           // Begin path.
    .m({x:0,y:0})                  // Move to point.
    .q({x1:200,y1:200,x:400,y:0})  // Quadratic bezier curve segment.
    .stroke()                      // Stroke path.
```
<a name="g2+c"></a>

### g2.c() ⇒ <code>object</code>
Create cubic bezier curve to point.

**Kind**: instance method of [<code>g2</code>](#g2)  
**Returns**: <code>object</code> - g2  

| Type | Description |
| --- | --- |
| <code>object</code> | cubic curve arguments object. |

**Properties**

| Name | Type | Description |
| --- | --- | --- |
| x1 | <code>number</code> | x coordinate of first control point. |
| y1 | <code>number</code> | y coordinate of first control point. |
| x2 | <code>number</code> | x coordinate of second control point. |
| y2 | <code>number</code> | y coordinate of second control point. |
| x | <code>number</code> | x coordinate of target point. |
| y | <code>number</code> | y coordinate of target point. |

**Example**  
```js
g2().p()                        // Begin path.
    .m({x:0,y:100})             // Move to point.
    .c({x1:100,y1:200,x2:200,y2:0,x:400,y:100}) // Create cubic bezier curve.
    .stroke()                   // Stroke path.
    .exe(ctx);                  // Render to canvas context.
```
<a name="g2+a"></a>

### g2.a() ⇒ <code>object</code>
Draw arc with angular range to target point.

**Kind**: instance method of [<code>g2</code>](#g2)  
**Returns**: <code>object</code> - g2  

| Type | Description |
| --- | --- |
| <code>object</code> | arc arguments object. |

**Properties**

| Name | Type | Description |
| --- | --- | --- |
| dw | <code>number</code> | angular range in radians. |
| x | <code>number</code> | x coordinate of target point. |
| y | <code>number</code> | y coordinate of target point. |

**Example**  
```js
g2().p()            // Begin path.
    .m({x:50,y:50})       // Move to point.
    .a({dw:2,x:300,y:100})   // Create arc segment.
    .stroke()       // Stroke path.
    .exe(ctx);      // Render to canvas context.
```
<a name="g2+stroke"></a>

### g2.stroke() ⇒ <code>object</code>
Stroke the current path or path object.

**Kind**: instance method of [<code>g2</code>](#g2)  
**Returns**: <code>object</code> - g2  

| Type | Description |
| --- | --- |
| <code>object</code> | stroke arguments object. |

**Properties**

| Name | Type | Description |
| --- | --- | --- |
| [d] | <code>string</code> | SVG path definition string. Current path is ignored then. |

<a name="g2+fill"></a>

### g2.fill() ⇒ <code>object</code>
Fill the current path or path object.

**Kind**: instance method of [<code>g2</code>](#g2)  
**Returns**: <code>object</code> - g2  

| Type | Description |
| --- | --- |
| <code>object</code> | fill arguments object. |

**Properties**

| Name | Type | Description |
| --- | --- | --- |
| [d] | <code>string</code> | SVG path definition string. Current path is ignored then. |

<a name="g2+drw"></a>

### g2.drw() ⇒ <code>object</code>
Shortcut for stroke and fill the current path or path object.
In case of shadow style, only the path interior creates shadow, not also the path contour.

**Kind**: instance method of [<code>g2</code>](#g2)  
**Returns**: <code>object</code> - g2  

| Type | Description |
| --- | --- |
| <code>object</code> | drw arguments object. |

**Properties**

| Name | Type | Description |
| --- | --- | --- |
| [d] | <code>string</code> | SVG path definition string.  Current path is ignored then. |

<a name="g2+del"></a>

### g2.del() ⇒ <code>object</code>
Delete all commands beginning from `idx` to end of command queue.

**Kind**: instance method of [<code>g2</code>](#g2)  
**Returns**: <code>object</code> - g2  
<a name="g2+ins"></a>

### g2.ins() ⇒ <code>object</code>
Call function between commands of the command queue.

**Kind**: instance method of [<code>g2</code>](#g2)  
**Returns**: <code>object</code> - g2  

| Type | Description |
| --- | --- |
| <code>function</code> | ins argument function. |

**Example**  
```js
const node = {
     fill:'lime',
     g2() { return g2().cir({x:160,y:50,r:15,fs:this.fill,lw:4,sh:[8,8,8,"gray"]}) }
};
let color = 'red';
g2().cir({x:40,y:50,r:15,fs:color,lw:4,sh:[8,8,8,"gray"]})   // draw red circle.
    .ins(()=>{color='green'})                                // color is now green.
    .cir({x:80,y:50,r:15,fs:color,lw:4,sh:[8,8,8,"gray"]})   // draw green circle.
    .ins((g) =>                                              // draw orange circle
         g.cir({x:120, y:50, r:15, fs:'orange', lw:4,sh:[8,8,8,"gray"]}))
    .ins(node)                                               // draw node.
    .exe(ctx)                                                // render to canvas context.
```
<a name="g2+exe"></a>

### g2.exe(ctx) ⇒ <code>object</code>
Execute g2 commands. It does so automatically and recursively with 'use'ed commands.

**Kind**: instance method of [<code>g2</code>](#g2)  
**Returns**: <code>object</code> - g2  

| Param | Type | Description |
| --- | --- | --- |
| ctx | <code>object</code> | Context. |

